import { Injectable } from '@nestjs/common';
import { InjectEntityManager, InjectRepository } from '@nestjs/typeorm';
import { MaterialsInventoryEntity } from './materials_inventory.entity';
import { EntityManager, Repository } from 'typeorm';
import {
  MaterialsInventoryDto,
  MaterialsInventoryExportDto,
  MaterialsInventoryQueryDto,
  MaterialsInventoryUpdateDto
} from './materials_inventory.dto';
import { Pagination } from '~/helper/paginate/pagination';
import { FastifyReply } from 'fastify';
import { paginate } from '~/helper/paginate';
import * as ExcelJS from 'exceljs';
@Injectable()
export class MaterialsInventoryService {
  constructor(
    @InjectEntityManager() private entityManager: EntityManager,
    @InjectRepository(MaterialsInventoryEntity)
    private materialsInventoryRepository: Repository<MaterialsInventoryEntity>
  ) {}

  /**
   * 导出原材料盘点表
   */
  async exportMaterialsInventoryCheck(dto: MaterialsInventoryExportDto, res: FastifyReply) {
    const workbook = new ExcelJS.Workbook();
    // 创建一个工作表
    const sheet = workbook.addWorksheet('CDKEY');
    // 设置表头
    sheet.columns = [
      { header: '名称', key: 'name', width: 32 },
      { header: '创建人', key: 'create_user', width: 32 },
      { header: '创建时间', key: 'create_time', width: 32 }
    ];
    const data = [
      {
        name: '名称',
        create_user: 'admin',
        create_time: '2023-05-18 12:00:00'
      }
    ];
    data.forEach(item => {
      sheet.addRow({
        name: item.name,
        create_user: item.create_user,
        create_time: item.create_time
      });
    });

    const buffer = await workbook.xlsx.writeBuffer();
    res
      .header('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
      .header(
        'Content-Disposition',
        `attachment; filename="${encodeURIComponent('导出_excel' + new Date().getTime() + '.xlsx')}"`
      )
      .send(buffer);
  }

  /**
   * 查询所有盘点信息
   */
  async findAll({
    page,
    pageSize
  }: MaterialsInventoryQueryDto): Promise<Pagination<MaterialsInventoryEntity>> {
    const queryBuilder = this.materialsInventoryRepository
      .createQueryBuilder('materialsInventory')
      .where('materialsInventory.isDelete = 0');

    return paginate<MaterialsInventoryEntity>(queryBuilder, {
      page,
      pageSize
    });
  }

  /**
   * 新增
   */
  async create(dto: MaterialsInventoryDto): Promise<void> {
    await this.materialsInventoryRepository.insert(dto);
  }

  /**
   * 更新
   */
  async update(id: number, data: Partial<MaterialsInventoryUpdateDto>): Promise<void> {
    await this.entityManager.transaction(async manager => {
      await manager.update(MaterialsInventoryEntity, id, {
        ...data
      });
    });
  }

  /**
   * 删除
   */
  async delete(id: number): Promise<void> {
    // 合同比较重要，做逻辑删除
    await this.materialsInventoryRepository.update(id, { isDelete: 1 });
  }

  /**
   * 获取单个合同信息
   */
  async info(id: number) {
    const info = await this.materialsInventoryRepository
      .createQueryBuilder('materialsInventory')
      .where({
        id
      })
      .andWhere('materialsInventory.isDelete = 0')
      .getOne();
    return info;
  }
}
